import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:nfc/student_edit_page.dart';
import 'package:open_file/open_file.dart';
import 'package:pdf/widgets.dart' as pw;
import 'package:path_provider/path_provider.dart';
import 'config.dart';
import 'dart:io';
import 'nfc_service.dart';

class DataPage extends StatefulWidget {
  final String googleSheetsLink;

  const DataPage({Key? key, required this.googleSheetsLink}) : super(key: key);

  @override
  _DataPageState createState() => _DataPageState();
}

class _DataPageState extends State<DataPage> {
  List<List<dynamic>> _sheetData = [];
  late bool _isLoading;

  @override
  void initState() {
    super.initState();
    _isLoading = true;
    _loadSheetData();
  }

  Future<void> _loadSheetData() async {
    RegExp regex = RegExp(
        r'https:\/\/docs\.google\.com\/spreadsheets\/d\/([a-zA-Z0-9_-]+)\/(edit|pubhtml).*');
    Match? match = regex.firstMatch(widget.googleSheetsLink);

    if (match != null) {
      String sheetId = match.group(1)!;
      String apiUrl =
          'https://sheets.googleapis.com/v4/spreadsheets/$sheetId/values/${AppConfig.sheetName}?key=${AppConfig.apiKey}';

      try {
        final response = await http.get(Uri.parse(apiUrl));

        if (response.statusCode == 200) {
          final Map<String, dynamic> data = json.decode(response.body);
          List<List<dynamic>> values = [];

          if (data['values'] == null) {
            setState(() {
              _isLoading = false;
            });
            return;
          }

          for (final entry in data['values']) {
            values.add(List.from(entry));
          }

          setState(() {
            _sheetData = values;
          });
        } else {
          print(
              'Erreur lors de la récupération des données du Google Sheets. Statut : ${response.statusCode}');
        }
      } catch (e) {
        print(
            'Erreur lors de la récupération des données du Google Sheets : $e');
      }
    }
  }

  Future<void> generatePdf(List<List<dynamic>> sheetData) async {
    final pdf = pw.Document();

    final List<pw.TableRow> rows = [];

    for (final row in sheetData) {
      final List<pw.Widget> rowWidgets = row.map((cell) => pw.Text('$cell')).toList();
      rows.add(pw.TableRow(children: rowWidgets));
    }

    pdf.addPage(pw.Page(
      build: (pw.Context context) {
        return pw.Table(
          columnWidths: {
            for (int i = 0; i < sheetData[0].length; i++)
              if (i < 2) i: const pw.FixedColumnWidth(70.0)
              else if (i == 5) i: const pw.FixedColumnWidth(150.0),
          },
          children: rows,
        );
      },
    ));

    //final file = File('google_sheets_data.pdf');
    //await file.writeAsBytes(await pdf.save());

    final directory = await getExternalStorageDirectory();
    final file = File('${directory!.path}/google_sheets_data.pdf');
    await file.writeAsBytes(await pdf.save());

    OpenFile.open(file.path);
  }

  void _handleNFCRead(String? nfcData) {
    if (nfcData != null) {
      // Affichage des données NFC lues
      showDialog(
        context: context,
        builder: (context) => AlertDialog(
          title: const Text('Données NFC'),
          content: Text('Données lues: $nfcData'),
          actions: <Widget>[
            TextButton(
              onPressed: () => Navigator.of(context).pop(),
              child: const Text('OK'),
            ),
          ],
        ),
      );
    } else {
      // Affichage d'une erreur ou d'une indication que le tag est vide
      showDialog(
        context: context,
        builder: (context) => AlertDialog(
          title: const Text('Erreur ou Tag NFC Vide'),
          content: const Text("Aucune donnée n'a été lue ou une erreur s'est produite lors de la lecture."),
          actions: <Widget>[
            TextButton(
              onPressed: () => Navigator.of(context).pop(),
              child: const Text('OK'),
            ),
          ],
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Page de données'),
        backgroundColor: Colors.deepPurple,
        centerTitle: true,
      ),
      body: SingleChildScrollView(
        child: Padding(
          padding: const EdgeInsets.all(16.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            children: <Widget>[
              if (_sheetData.isNotEmpty)
                Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    const Text(
                      'Contenu du Google Sheets :',
                      style:
                      TextStyle(fontSize: 18.0, fontWeight: FontWeight.bold, color: Colors.deepPurple),
                    ),
                    const SizedBox(height: 10.0),
                    SingleChildScrollView(
                      child: Column(
                        children: _sheetData.asMap().entries.map((entry) {
                          // Ignorer la première ligne (l'en-tête)
                          if (entry.key == 0) return SizedBox(); // Ignorer la première ligne (l'en-tête)

                          // Récupérer les données de la ligne actuelle
                          List<dynamic> row = entry.value;

                          // Assurez-vous que chaque ligne a au moins 5 éléments
                          if (row.length < 5) return SizedBox(); // Si la ligne n'a pas suffisamment d'éléments, ne rien afficher

                          // Décomposition de la liste en variables pour plus de clarté
                          String nom = row[0];
                          String prenom = row[1];
                          String numeroEtudiant = row[2];
                          String numeroNFC = row[3];
                          String presence = row[4];

                          return Card(
                            margin: EdgeInsets.symmetric(vertical: 5.0, horizontal: 10.0),
                            child: ListTile(
                              title: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  // Nom et prénom en gras
                                  Text(
                                    '$nom $prenom',
                                    style: TextStyle(fontWeight: FontWeight.bold),
                                  ),
                                  SizedBox(height: 5.0), // Espacement entre le nom/prénom et les autres informations
                                  // Informations supplémentaires
                                  Text('Numéro étudiant: $numeroEtudiant'),
                                  Text('Numéro NFC: $numeroNFC'),
                                  Text('Présent: $presence'),
                                ],
                              ),
                            ),
                          );
                        }).toList(),
                      ),
                    ),

                    const SizedBox(height: 20.0),
                    Wrap(
                      alignment: WrapAlignment.center, // Aligner les boutons au centre
                      spacing: 20.0,
                      runSpacing: 10.0,// Espace entre les boutons
                      children: <Widget>[
                        ElevatedButton(
                          onPressed: () async {
                            await generatePdf(_sheetData);
                          },
                          style: ElevatedButton.styleFrom(
                            primary: Colors.deepPurple,
                            onPrimary: Colors.white,
                          ),
                          child: const Text('Générer PDF'),
                        ),
                        ElevatedButton(
                          onPressed: () async {
                            // Créez une instance de NFCService
                            NFCService nfcService = NFCService(this.context);

                            await nfcService.readNFC(context, _handleNFCRead, widget.googleSheetsLink);
                          },
                          style: ElevatedButton.styleFrom(
                            primary: Colors.deepPurple,
                            onPrimary: Colors.white,
                          ),
                          child: const Text('Lire la leocarte'),
                        ),
                        ElevatedButton(
                          onPressed: () {
                            Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (context) => StudentEditPage(googleSheetsLink: widget.googleSheetsLink,codeNfc: ''),
                              ),
                            );
                          },
                          style: ElevatedButton.styleFrom(
                            primary: Colors.deepPurple,
                            onPrimary: Colors.white,
                          ),
                          child: const Text('Ajouter un étudiant'),
                        ),
                      ],
                    ),
                  ],
                )
              else if (_isLoading)
                const Text(
                  'Chargement des données..',
                  style: TextStyle(fontSize: 16.0),
                )
              else if (_sheetData.isEmpty)
                  const Text(
                    'Aucune donnée disponible.',
                    style: TextStyle(fontSize: 16.0),
                  ),
            ],
          ),
        ),
      ),
    );
  }
}
