import 'package:flutter/material.dart';
import 'package:nfc/student_edit_page.dart';
import 'package:nfc_manager/nfc_manager.dart';
import 'package:googleapis_auth/auth_io.dart' as auth;
import 'package:googleapis/sheets/v4.dart' as sheets;
import 'dart:io' show File;

import 'config.dart';

class NFCService {
  String? _spreadsheetId;
  BuildContext context;

  NFCService(this.context);

  void _extractSpreadsheetId(String googleSheetsLink) {
    final regExp = RegExp(r'/spreadsheets/d/([a-zA-Z0-9-_]+)');
    final matches = regExp.firstMatch(googleSheetsLink);
    if (matches != null && matches.groupCount >= 1) {
      _spreadsheetId = matches.group(1);
    } else {
      print("L'URL de la feuille Google Sheets est invalide");
    }
  }

  Future<void> readNFC(BuildContext context, void Function(String?) onRead, String googleSheetsLink) async {
    try {
      bool isAvailable = await NfcManager.instance.isAvailable();
      if (!isAvailable) {
        ScaffoldMessenger.of(context).showSnackBar(SnackBar(
            content: Text("NFC n'est pas disponible sur cet appareil")));
        onRead(null);
        return;
      }

      await NfcManager.instance.startSession(
        onDiscovered: (NfcTag tag) async {
          try {
            // Récupérez les données de l'identifiant du tag
            String tagId = tag.data['isodep']['identifier'].join();

            // Appeler updatePresenceInGoogleSheet
            bool isFound = await updatePresenceInGoogleSheet(googleSheetsLink, tagId);

            // Arrêtez la session NFC
            NfcManager.instance.stopSession();

            // Appelez la fonction de rappel avec la valeur lue
            if (isFound) {
              onRead(tagId);
            }
          } catch (e) {
            ScaffoldMessenger.of(context).showSnackBar(
                SnackBar(content: Text("Erreur lors de la lecture NFC: $e")));
            onRead(null);
          }
        },
      );
    } catch (e) {
      ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text("Erreur lors de la lecture NFC: $e")));
      onRead(null);
    }
  }

  Future<bool> updatePresenceInGoogleSheet(String googleSheetsLink, String tagId) async {
    _extractSpreadsheetId(googleSheetsLink);
    var isFound = false;


    if (_spreadsheetId == null) {
      print("Erreur: Impossible de récupérer l'identifiant de la feuille Google Sheets.");
      return isFound;
    }

    final jsonContent = await AppConfig.loadJson();
    final accountCredentials = auth.ServiceAccountCredentials.fromJson(jsonContent);

    final scopes = [sheets.SheetsApi.spreadsheetsScope];

    final authClient = await auth.clientViaServiceAccount(
        accountCredentials, scopes);
    var api = sheets.SheetsApi(authClient);
    var range = '${AppConfig.sheetName}!A:E';

    try {
      var response = await api.spreadsheets.values.get(_spreadsheetId!, range);
      var values = response.values;

      if (values == null) {
        print("Aucune donnée disponible dans le Google Sheets.");
        return isFound;
      }
      print("Données récupérées avec succès depuis le Google Sheets.");

      for (var row in values) {
        if (row.length >= 4 && row[3] == tagId) {
          // Mettez à jour la colonne "Présence" à "oui"
          row[4] = "oui";
          var updateRange = '${AppConfig.sheetName}!A${values.indexOf(row) + 1}:E${values.indexOf(row) + 1}';
          var valueRange = sheets.ValueRange.fromJson({"values": [row]});
          await api.spreadsheets.values.update(valueRange, _spreadsheetId!, updateRange, valueInputOption: 'RAW');
          isFound = true;
          break;
        }
      }
      if (!isFound) {
        print("Aucune correspondance trouvée pour le tag NFC dans le Google Sheets.");
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) => StudentEditPage(googleSheetsLink: googleSheetsLink, codeNfc: tagId),
          ),
        );
      }

      return isFound;
    } catch (e) {
      print("Erreur lors de la mise à jour de la présence dans le Google Sheets: $e");
      return isFound;
    }
  }
}
